Method: Zafu::Process::Context#open_node_context
- Defined in:
- lib/zafu/process/context.rb
#open_node_context(finder, cont = {}) ⇒ Object
This should be called when we enter a new node context so that the proper hooks are triggered (insertion of contextual variables).
173 174 175 176 177 178 179 |
# File 'lib/zafu/process/context.rb', line 173 def open_node_context(finder, cont = {}) sub_context = node_context_vars(finder).merge(cont) with_context(sub_context) do yield end end |